package com.example.mobileapp.ui.screens

import android.annotation.SuppressLint
import androidx.compose.foundation.BorderStroke
import androidx.compose.foundation.Image
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxHeight
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material3.ButtonDefaults
import androidx.compose.material3.Card
import androidx.compose.material3.OutlinedButton
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.livedata.observeAsState
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.text.font.Font
import androidx.compose.ui.text.font.FontFamily
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.hilt.navigation.compose.hiltViewModel
import com.example.mobileapp.data.sharedpref.PreferencesManager
import com.example.mobileapp.R
import com.example.mobileapp.data.db.models.OrderProductWithProduct
import com.example.mobileapp.data.db.models.OrderWithProducts
import com.example.mobileapp.viewmodels.CartViewModel
import com.example.mobileapp.ui.widgets.Header

@Composable
fun CartScreen(){
    Column (
        modifier = Modifier.fillMaxSize(),
        horizontalAlignment = Alignment.CenterHorizontally,
    ) {
        Header(Color(212, 206, 203))
        CartCard()
    }
}

@SuppressLint("CoroutineCreationDuringComposition")
@Composable
fun CartCard(){
    val sharedPref = PreferencesManager(LocalContext.current)
    val searchToken = sharedPref.getData("token", "")
    val cartViewModel: CartViewModel = hiltViewModel<CartViewModel>()
    cartViewModel.getCart(searchToken)
    val cart: OrderWithProducts? = cartViewModel.cart.observeAsState().value

    Column(
        modifier = Modifier
            .fillMaxSize()
            .background(Color(212, 206, 203)),
        horizontalAlignment = Alignment.CenterHorizontally,
        ) {
        Column(
            modifier = Modifier
                .fillMaxHeight()
                .fillMaxWidth(0.9f)
                .background(Color.White),
            horizontalAlignment = Alignment.CenterHorizontally,
        ) {
            Text(
                text = "Корзина",
                color = Color.Black,
                fontFamily = FontFamily(Font(R.font.nunito_extrabold_italic)),
                fontSize = 34.sp,
                modifier = Modifier
                    .fillMaxWidth()
                    .padding(top = 10.dp),
                textAlign = TextAlign.Center,
            )

            if (cart != null && cart.orderWithProducts.size != 0) {
                LazyColumn(
                    modifier = Modifier
                        .fillMaxWidth()
                ){
                    items(cart.orderWithProducts.size) { index ->
                        CartProductCard(cartViewModel, cart.orderWithProducts[index])
                    }
                }
                Card(
                    shape = RoundedCornerShape(20.dp),
                    border = BorderStroke(3.dp, Color(222,161,69)),
                    modifier = Modifier
                        .padding(top = 15.dp, start = 5.dp, end = 5.dp)
                ){
                    Row(
                        modifier = Modifier
                            .fillMaxWidth()
                            .background(Color.White),
                        horizontalArrangement = Arrangement.SpaceBetween,
                        verticalAlignment = Alignment.CenterVertically,
                    ){
                        Text(
                            text = "Итого: " + cart.order.price.toString() + " р.",
                            color = Color.Black,
                            fontFamily = FontFamily(Font(R.font.nunito_extrabold_italic)),
                            fontSize = 20.sp,
                            textAlign = TextAlign.Center,
                            modifier = Modifier.padding(start = 5.dp),
                        )

                        OutlinedButton(
                            onClick = {
                                cartViewModel.makeOrder(searchToken)
                            },
                            border = BorderStroke(4.dp, Color(222,161,69)),
                            shape = RoundedCornerShape(20.dp),
                            colors = ButtonDefaults.outlinedButtonColors(Color(255,186,83)),
                            modifier = Modifier.padding(5.dp)
                        ) {
                            Text(text = "Заказать", fontSize = 20.sp, color = Color.White)
                        }
                    }
                }
            }
        }
    }
}

@Composable
fun CartProductCard(
    cartViewModel: CartViewModel,
    product: OrderProductWithProduct
){
    val sharedPref = PreferencesManager(LocalContext.current)
    val searchToken = sharedPref.getData("token", "")
    var amountProduct by remember { mutableStateOf(product.orderProduct.amount)}

    Card(
        shape = RoundedCornerShape(20.dp),
        border = BorderStroke(3.dp, Color(222,161,69)),
        modifier = Modifier
            .fillMaxSize()
            .padding(top = 15.dp, start = 5.dp, end = 5.dp)
    ){
        Row(
            modifier = Modifier
                .fillMaxSize()
                .background(Color.White),
            verticalAlignment = Alignment.CenterVertically,
            horizontalArrangement = Arrangement.SpaceEvenly,
        ){
            CartProductImageCard(product.product.categoryId)

            Column (
                horizontalAlignment = Alignment.CenterHorizontally,
                verticalArrangement = Arrangement.SpaceAround
            ) {
                Text(
                    text = product.product.name,
                    color = Color.Black,
                    fontFamily = FontFamily(Font(R.font.nunito_extrabold_italic)),
                    fontSize = 17.sp,
                    textAlign = TextAlign.Center,
                )
                Text(
                    text = product.product.price.toString() + " р.",
                    color = Color.Black,
                    fontFamily = FontFamily(Font(R.font.nunito_extrabold_italic)),
                    fontSize = 17.sp,
                    textAlign = TextAlign.Center,
                )
                Row(
                    verticalAlignment = Alignment.CenterVertically,
                    horizontalArrangement = Arrangement.Center,
                    modifier = Modifier
                        .fillMaxWidth(0.8f)
                ){
                    OutlinedButton(
                        modifier = Modifier
                            .fillMaxHeight(),
                        onClick = {
                            cartViewModel.removeProductFromOrder(searchToken, product.product.id!!)
                            amountProduct -= 1
                        },
                        border = BorderStroke(4.dp, Color(222,161,69)),
                        shape = RoundedCornerShape(topStart = 30.dp, bottomStart = 30.dp),
                        colors = ButtonDefaults.outlinedButtonColors(Color(255,186,83)),
                    ) {
                        Text(text = "-", fontSize = 20.sp, color = Color.Black)
                    }

                    Card(
                        modifier = Modifier
                            .fillMaxHeight(),
                        border = BorderStroke(3.dp, Color(222,161,69)),
                        shape = RoundedCornerShape(0.dp),
                    ){
                        Text(
                            modifier = Modifier
                                .background(Color.White)
                                .fillMaxHeight()
                                .padding(
                                    start = 20.dp,
                                    end = 20.dp,
                                    top = 7.dp,
                                    bottom = 7.dp
                                ),
                            text = amountProduct.toString(),
                            color = Color.Black,
                            fontFamily = FontFamily(Font(R.font.nunito_bold)),
                            fontSize = 20.sp,
                            textAlign = TextAlign.Center,
                        )
                    }

                    OutlinedButton(
                        modifier = Modifier
                            .fillMaxHeight(),
                        onClick = {
                            cartViewModel.addProductToOrder(searchToken, product.product.id!!)
                            amountProduct += 1
                        },
                        border = BorderStroke(4.dp, Color(222,161,69)),
                        shape = RoundedCornerShape(topEnd = 30.dp, bottomEnd = 30.dp),
                        colors = ButtonDefaults.outlinedButtonColors(Color(255,186,83)),
                    ) {
                        Text(text = "+", fontSize = 20.sp, color = Color.Black)
                    }
                }
            }
        }
    }
}

@Composable
fun CartProductImageCard(categoryId : Int){
    if (categoryId == 1){
        Image(
            painterResource(
                id = R.drawable.burger
            ),
            contentDescription = null,
            modifier = Modifier
                .size(100.dp, 100.dp)
                .padding(5.dp),
        )
    }
    if (categoryId == 2){
        Image(
            painterResource(
                id = R.drawable.potato
            ),
            contentDescription = null,
            modifier = Modifier
                .size(100.dp, 100.dp)
                .padding(5.dp),
        )
    }
    if (categoryId == 3){
        Image(
            painterResource(
                id = R.drawable.pepsi
            ),
            contentDescription = null,
            modifier = Modifier
                .size(100.dp, 100.dp)
                .padding(5.dp),
        )
    }
}