package com.example.mobileapp.ui.screens

import androidx.compose.foundation.BorderStroke
import androidx.compose.foundation.Image
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxHeight
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material3.ButtonDefaults
import androidx.compose.material3.Card
import androidx.compose.material3.OutlinedButton
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.collectAsState
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.text.font.Font
import androidx.compose.ui.text.font.FontFamily
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.paging.compose.collectAsLazyPagingItems
import androidx.paging.compose.itemKey
import com.example.mobileapp.data.sharedpref.PreferencesManager
import com.example.mobileapp.R
import com.example.mobileapp.data.db.models.Category
import com.example.mobileapp.data.db.models.Product
import com.example.mobileapp.viewmodels.MenuViewModel
import com.example.mobileapp.ui.widgets.Header

@Composable
fun MenuScreen(){
    Column (
        modifier = Modifier.fillMaxSize(),
        horizontalAlignment = Alignment.CenterHorizontally,
    ) {
        Header(Color(212, 206, 203))
        MenuCard()
    }
}

@Composable
fun MenuCard(){
    val sharedPref = PreferencesManager(LocalContext.current)
    val searchToken = sharedPref.getData("token", "")
    val menuViewModel: MenuViewModel = hiltViewModel<MenuViewModel>()
    val categoryListUi = menuViewModel.getCategoriesList(searchToken).collectAsLazyPagingItems()

    Column(
        modifier = Modifier
            .fillMaxSize()
            .background(Color(212, 206, 203)),
        horizontalAlignment = Alignment.CenterHorizontally,

        ) {
        Column(
            modifier = Modifier
                .fillMaxHeight()
                .fillMaxWidth(0.9f)
                .background(Color.White),

            horizontalAlignment = Alignment.CenterHorizontally,
        ) {
            LazyColumn() {
                items(
                    count = categoryListUi.itemCount,
                    key = categoryListUi.itemKey()
                ){ index ->
                    MenuCategoryProductsCard(menuViewModel, categoryListUi[index]!!, searchToken)
                }
            }
        }
    }
}

@Composable
fun MenuCategoryProductsCard(
    menuViewModel: MenuViewModel,
    category: Category,
    searchToken: String
){
    val productListUiState = menuViewModel.getProductsByCategoryId(category.id!!, searchToken).collectAsState(initial = listOf())

    if (productListUiState.value != null){
        Text(
            text = category.name,
            color = Color.Black,
            fontFamily = FontFamily(Font(R.font.nunito_extrabold_italic)),
            fontSize = 34.sp,
            modifier = Modifier
                .fillMaxWidth()
                .padding(top = 10.dp),
            textAlign = TextAlign.Center,
        )

        Column {
            productListUiState.value!!.forEach{product ->
                MenuProductsCard(menuViewModel, product, category, searchToken)
            }
        }
    }
}

@Composable
fun MenuProductsCard(
    menuViewModel: MenuViewModel,
    product: Product,
    category: Category,
    searchToken: String
){
    val sharedPref = PreferencesManager(LocalContext.current)
    val userId = sharedPref.getData("userId", "").toInt()
    Card(
        shape = RoundedCornerShape(20.dp),
        border = BorderStroke(3.dp, Color(222,161,69)),
        modifier = Modifier
            .fillMaxSize()
            .padding(top = 15.dp, start = 5.dp, end = 5.dp)
    ){
        Row(
            modifier = Modifier
                .fillMaxSize()
                .background(Color.White),
            verticalAlignment = Alignment.CenterVertically,
            horizontalArrangement = Arrangement.SpaceEvenly,
        ){
            MenuProductImageCard(category.name)

            Column (
                horizontalAlignment = Alignment.CenterHorizontally,
                verticalArrangement = Arrangement.SpaceAround
            ) {
                Text(
                    text = product.name,
                    color = Color.Black,
                    fontFamily = FontFamily(Font(R.font.nunito_extrabold_italic)),
                    fontSize = 17.sp,
                    textAlign = TextAlign.Center,
                )
                Row(
                    verticalAlignment = Alignment.CenterVertically,
                    horizontalArrangement = Arrangement.spacedBy(10.dp),
                ){
                    Text(
                        text = product.price.toString() + " р.",
                        color = Color.Black,
                        fontFamily = FontFamily(Font(R.font.nunito_extrabold_italic)),
                        fontSize = 17.sp,
                        textAlign = TextAlign.Center,
                    )
                    OutlinedButton(
                        onClick = {
                            menuViewModel.addProductToCart(product.id!!, userId, searchToken)
                        },
                        border = BorderStroke(4.dp, Color(222,161,69)),
                        shape = RoundedCornerShape(30.dp),
                        colors = ButtonDefaults.outlinedButtonColors(Color(255,186,83)),
                    ) {
                        Text(text = "+", fontSize = 20.sp, color = Color.Black)
                    }
                }
            }
        }
    }
}

@Composable
fun MenuProductImageCard(categoryName : String){
    if (categoryName == "Бургеры"){
        Image(
            painterResource(
                id = R.drawable.burger
            ),
            contentDescription = null,
            modifier = Modifier
                .size(100.dp, 100.dp)
                .padding(5.dp),
        )
    }
    if (categoryName == "Картошка"){
        Image(
            painterResource(
                id = R.drawable.potato
            ),
            contentDescription = null,
            modifier = Modifier
                .size(100.dp, 100.dp)
                .padding(5.dp),
        )
    }
    if (categoryName == "Напитки"){
        Image(
            painterResource(
                id = R.drawable.pepsi
            ),
            contentDescription = null,
            modifier = Modifier
                .size(100.dp, 100.dp)
                .padding(5.dp),
        )
    }
}