package com.example.mobileapp.ui.screens

import android.annotation.SuppressLint
import androidx.compose.foundation.BorderStroke
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxHeight
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material3.ButtonDefaults
import androidx.compose.material3.Card
import androidx.compose.material3.OutlinedButton
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.text.font.Font
import androidx.compose.ui.text.font.FontFamily
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.navigation.NavController
import androidx.paging.compose.collectAsLazyPagingItems
import androidx.paging.compose.itemKey
import com.example.mobileapp.data.sharedpref.PreferencesManager
import com.example.mobileapp.R
import com.example.mobileapp.data.db.models.OrderWithProducts
import com.example.mobileapp.ui.navigation.Screens
import com.example.mobileapp.viewmodels.ProfileViewModel
import com.example.mobileapp.ui.widgets.Header
import kotlinx.coroutines.launch

@Composable
fun ProfileScreen(navController: NavController){
    Column (
        modifier = Modifier.fillMaxSize(),
        horizontalAlignment = Alignment.CenterHorizontally,
    ) {
        Header(Color(212, 206, 203))
        ProfileCard(navController)
    }
}

@SuppressLint("CoroutineCreationDuringComposition")
@Composable
fun ProfileCard(navController: NavController){
    val sharedPref = PreferencesManager(LocalContext.current)
    val searchToken = sharedPref.getData("token", "")
    val profileViewModel: ProfileViewModel = hiltViewModel<ProfileViewModel>()
    val activeOrdersListUiState = profileViewModel.getActiveOrders(searchToken, sharedPref.getData("userId", "-1").toInt())?.collectAsLazyPagingItems()

    Column(
        modifier = Modifier
            .fillMaxSize()
            .background(Color(212, 206, 203)),
        horizontalAlignment = Alignment.CenterHorizontally,
        ) {
        Column(
            modifier = Modifier
                .fillMaxHeight()
                .fillMaxWidth(0.9f)
                .background(Color.White),
            horizontalAlignment = Alignment.CenterHorizontally,
        ) {
            OutlinedButton(
                modifier = Modifier
                    .fillMaxWidth()
                    .padding(top = 15.dp, start = 5.dp, end = 5.dp),
                onClick = {
                    navController.navigate(Screens.AuthorizationScreen.name)
                    profileViewModel.logout(sharedPref)
                },
                border = BorderStroke(4.dp, Color(255,86,86)),
                shape = RoundedCornerShape(20.dp),
                colors = ButtonDefaults.outlinedButtonColors(Color(255,86,86)),
            ) {
                Text(
                    text = "Выйти из аккаунта",
                    fontFamily = FontFamily(Font(R.font.nunito_extrabold_italic)),
                    fontSize = 30.sp,
                    color = Color.White
                )
            }

            Text(
                text = "Активные заказы",
                color = Color.Black,
                fontFamily = FontFamily(Font(R.font.nunito_extrabold_italic)),
                fontSize = 34.sp,
                modifier = Modifier
                    .fillMaxWidth()
                    .padding(top = 10.dp),
                textAlign = TextAlign.Center,
            )


            if (activeOrdersListUiState != null){
                LazyColumn(
                    modifier = Modifier
                        .fillMaxWidth()
                ) {
                    items(
                        count = activeOrdersListUiState.itemCount,
                        key = activeOrdersListUiState.itemKey()
                    ){ index ->
                        ActiveOrderCard(activeOrdersListUiState[index]!!, profileViewModel)
                    }
                }
            }
        }
    }
}

@SuppressLint("CoroutineCreationDuringComposition")
@Composable
fun ActiveOrderCard(order: OrderWithProducts, profileViewModel: ProfileViewModel) {
    val scope = rememberCoroutineScope()
    val status = remember { mutableStateOf("")}
    scope.launch {
        status.value = profileViewModel.getOrderStatus(order.order.statusId!!)
    }
    Card(
        shape = RoundedCornerShape(20.dp),
        border = BorderStroke(3.dp, Color(222,161,69)),
        modifier = Modifier
            .padding(top = 15.dp, start = 5.dp, end = 5.dp)
    ) {
        Row(
            modifier = Modifier
                .fillMaxWidth()
                .background(Color.White),
            verticalAlignment = Alignment.CenterVertically,
            horizontalArrangement = Arrangement.SpaceEvenly,
        ) {
            Text(
                text = "№" + order.order.id.toString(),
                color = Color.Black,
                fontFamily = FontFamily(Font(R.font.nunito_extrabold_italic)),
                fontSize = 34.sp,
                textAlign = TextAlign.Center,
            )
            Text(
                text = "Статус: " + status.value,
                color = Color.Black,
                fontFamily = FontFamily(Font(R.font.nunito_extrabold_italic)),
                fontSize = 20.sp,
                textAlign = TextAlign.Center,
            )
        }
        order.orderWithProducts.forEach { orderProduct ->
            Row(
                modifier = Modifier
                    .fillMaxWidth()
                    .background(Color.White),
                verticalAlignment = Alignment.CenterVertically,
                horizontalArrangement = Arrangement.Center,
            ) {
                Text(
                    text = orderProduct.product.name + " " + orderProduct.product.price.toString() + "р. x" + orderProduct.orderProduct.amount.toString(),
                    color = Color.Gray,
                    fontFamily = FontFamily(Font(R.font.nunito_bold)),
                    fontSize = 20.sp,
                    textAlign = TextAlign.Center,
                )
            }
        }
        Row(
            modifier = Modifier
                .fillMaxWidth()
                .background(Color.White),
            verticalAlignment = Alignment.CenterVertically,
            horizontalArrangement = Arrangement.Start,
        ) {
            Text(
                text = "К оплате: " + order.order.price.toString() + " р.",
                modifier = Modifier.padding(start = 20.dp),
                color = Color.Black,
                fontFamily = FontFamily(Font(R.font.nunito_extrabold_italic)),
                fontSize = 20.sp,
                textAlign = TextAlign.Center,
            )
        }
    }
}