package com.example.mobileapp.ui.screens

import android.annotation.SuppressLint
import androidx.compose.foundation.BorderStroke
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxHeight
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material3.ButtonDefaults
import androidx.compose.material3.Card
import androidx.compose.material3.DropdownMenuItem
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.ExposedDropdownMenuBox
import androidx.compose.material3.ExposedDropdownMenuDefaults
import androidx.compose.material3.Icon
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.OutlinedButton
import androidx.compose.material3.Text
import androidx.compose.material3.TextField
import androidx.compose.material3.lightColorScheme
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateListOf
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.text.font.Font
import androidx.compose.ui.text.font.FontFamily
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.paging.compose.collectAsLazyPagingItems
import androidx.paging.compose.itemKey
import com.example.mobileapp.data.sharedpref.PreferencesManager
import com.example.mobileapp.R
import com.example.mobileapp.data.db.models.User
import com.example.mobileapp.viewmodels.UsersViewModel
import com.example.mobileapp.ui.widgets.Header

@Composable
fun UsersScreen() {
    Column (
        modifier = Modifier.fillMaxSize(),
        horizontalAlignment = Alignment.CenterHorizontally,
    ) {
        Header(Color(212, 206, 203))
        UsersCard()
    }
}

@SuppressLint("CoroutineCreationDuringComposition")
@Composable
fun UsersCard(){
    val sharedPref = PreferencesManager(LocalContext.current)
    val usersViewModel: UsersViewModel = hiltViewModel<UsersViewModel>()
    val searchToken = sharedPref.getData("token", "")
    val userListUiState = usersViewModel.getUsersList(searchToken).collectAsLazyPagingItems()

    Column(
        modifier = Modifier
            .fillMaxSize()
            .background(Color(212, 206, 203)),
        horizontalAlignment = Alignment.CenterHorizontally,

        ) {
        Column(
            modifier = Modifier
                .fillMaxHeight()
                .fillMaxWidth(0.9f)
                .background(Color.White),

            horizontalAlignment = Alignment.CenterHorizontally,
        ) {
            Text(
                text = "Редактирование пользователей",
                color = Color.Black,
                fontFamily = FontFamily(Font(R.font.nunito_extrabold_italic)),
                fontSize = 34.sp,
                modifier = Modifier
                    .fillMaxWidth()
                    .padding(top = 10.dp),
                textAlign = TextAlign.Center,
            )
            if (userListUiState != null && usersViewModel.allRoles.size != 0){
                LazyColumn(
                    modifier = Modifier
                        .fillMaxSize()
                ) {
                    items(
                        count = userListUiState.itemCount,
                        key = userListUiState.itemKey()
                    ){ index ->
                        if (userListUiState[index]!!.id != sharedPref.getData("userId", "-1").toInt()){
                            UserCard(userListUiState[index]!!, usersViewModel)
                        }
                    }
                }
            }
        }
    }
}

@SuppressLint("CoroutineCreationDuringComposition")
@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun UserCard (user: User, usersViewModel: UsersViewModel){
    val sharedPref = PreferencesManager(LocalContext.current)
    val searchToken = sharedPref.getData("token", "")
    val roleList = remember { mutableStateListOf(usersViewModel.allRoles)}
    var selectedRole by remember { mutableStateOf(usersViewModel.getUserRole(user.roleId))}
    var expanded by remember { mutableStateOf(false) }

    Card(
        shape = RoundedCornerShape(20.dp),
        border = BorderStroke(3.dp, Color(222,161,69)),
        modifier = Modifier
            .fillMaxSize()
            .padding(top = 15.dp, start = 5.dp, end = 5.dp)
    ){
        Row(
            modifier = Modifier
                .fillMaxSize()
                .background(Color.White),
            verticalAlignment = Alignment.CenterVertically,
            horizontalArrangement = Arrangement.SpaceEvenly,
        ){
            Column (
                horizontalAlignment = Alignment.CenterHorizontally,
                verticalArrangement = Arrangement.SpaceAround,
                modifier = Modifier.fillMaxWidth(0.4f)
            ) {
                Text(
                    text = "Логин: " + user.login,
                    color = Color.Black,
                    fontFamily = FontFamily(Font(R.font.nunito_extrabold_italic)),
                    fontSize = 17.sp,
                    textAlign = TextAlign.Center,
                    modifier = Modifier.padding(top = 5.dp)
                )
                Card(
                    shape = RoundedCornerShape(20.dp),
                    border = BorderStroke(1.dp, Color(222,161,69)),
                    modifier = Modifier
                        .fillMaxWidth()
                        .padding(bottom = 10.dp),
                ){
                    MaterialTheme(
                        colorScheme = lightColorScheme(
                            surfaceVariant= Color.White,
                            surface = Color.White,
                            onSurface = Color.Gray
                        )
                    ){
                        ExposedDropdownMenuBox(
                            expanded = expanded,
                            onExpandedChange = {
                                expanded = !expanded
                            },
                        ) {
                            TextField(
                                modifier = Modifier
                                    .menuAnchor()
                                    .fillMaxWidth(),
                                readOnly = true,
                                value = selectedRole,
                                onValueChange = {},
                                trailingIcon = { ExposedDropdownMenuDefaults.TrailingIcon(expanded = expanded) },
                            )

                            ExposedDropdownMenu(
                                expanded = expanded,
                                onDismissRequest = {
                                    expanded = false
                                },
                            ) {
                                roleList.first().forEach {selectionOption ->
                                    DropdownMenuItem(

                                        text = { Text(selectionOption.name) },
                                        onClick = {
                                            selectedRole = selectionOption.name
                                            expanded = false
                                        },
                                        contentPadding = ExposedDropdownMenuDefaults.ItemContentPadding,
                                    )
                                }
                            }
                        }
                    }
                }
            }
            OutlinedButton(
                onClick = {
                    usersViewModel.changeUserRole(user.id!!, selectedRole, searchToken)
                },
                border = BorderStroke(4.dp, Color(222,161,69)),
                shape = RoundedCornerShape(10.dp),
                colors = ButtonDefaults.outlinedButtonColors(Color(255,186,83)),

                ){
                Icon(
                    painterResource(id = R.drawable.icon_settings),
                    contentDescription = null,
                    Modifier.size(30.dp),
                    tint = Color.Black
                )
            }
            OutlinedButton(
                onClick = {
                    usersViewModel.deleteUser(user.id!!, searchToken)
                },
                border = BorderStroke(4.dp, Color(222,161,69)),
                shape = RoundedCornerShape(10.dp),
                colors = ButtonDefaults.outlinedButtonColors(Color(255,186,83)),

                ){
                Icon(
                    painterResource(id = R.drawable.icon_trash),
                    contentDescription = null,
                    Modifier.size(30.dp),
                    tint = Color.Black
                )
            }
        }
    }
}