package com.example.mobileapp.viewmodels

import androidx.compose.runtime.mutableStateListOf
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.example.mobileapp.data.api.models.CategoryRemote
import com.example.mobileapp.data.api.models.toCategory
import com.example.mobileapp.data.db.models.Category
import com.example.mobileapp.data.db.models.Product
import com.example.mobileapp.data.repositories.CategoryRepository
import com.example.mobileapp.data.repositories.ProductRepository
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.launch
import java.util.stream.Collectors
import javax.inject.Inject

@HiltViewModel
class ProductViewModel @Inject constructor(
    private val productRepository: ProductRepository,
    private val categoryRepository: CategoryRepository,
) : ViewModel() {
    private val _successState = MutableLiveData<Boolean?>()
    private val _categories = mutableStateListOf<Category>()
    val categories: List<Category>
        get() = _categories
    val successState: LiveData<Boolean?>
        get() = _successState

    private val _product = MutableLiveData<Product?>()
    val product: LiveData<Product?>
        get() = _product

    fun getProduct(productId: Int, token: String) {
        viewModelScope.launch {
            _product.postValue(productRepository.getProductById(productId, token))
        }
    }

    init {
        if (categories.size == 0){
            viewModelScope.launch {
                _categories.addAll(categoryRepository.getCategories().stream().map(CategoryRemote::toCategory).collect(Collectors.toList()))
            }
        }
    }

    fun getProductCategory(categoryId : Int) : String{
        return categories.find { it.id == categoryId }?.name ?: ""
    }

    fun addProduct(name: String, categoryName: String, price: String, token: String) {
        if (name.isEmpty() || name.length > 10 || name.length < 3){
            _successState.postValue(false)
            return
        }
        if (categoryName.isEmpty()){
            _successState.postValue(false)
            return
        }
        if (price.isEmpty() || price.toIntOrNull() == null || price.toInt() <= 0){
            _successState.postValue(false)
            return
        }

        viewModelScope.launch {
            productRepository.insert(
                Product(
                    null,
                    name,
                    categories.find { it.name == categoryName }!!.id!!,
                    price.toInt()
                ),
                token
            )
            _successState.postValue(true)
        }
    }

    fun updateProduct(id: Int, name: String, categoryName: String, price: String, token: String) {
        if (name.isEmpty() || name.length > 10 || name.length < 3){
            _successState.postValue(false)
            return
        }
        if (categoryName.isEmpty()){
            _successState.postValue(false)
            return
        }
        if (price.isEmpty() || price.toIntOrNull() == null || price.toInt() <= 0){
            _successState.postValue(false)
            return
        }

        viewModelScope.launch {
            productRepository.update(
                Product(
                    id,
                    name,
                    categories.find {it.name == categoryName}!!.id!!,
                    price.toInt()
                ),
                token
            )
            _successState.postValue(true)
        }
    }
}